"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServiceMethodSource = exports.createServiceSource = exports.createGrpcServiceSource = exports.createSource = exports.createNamedImports = exports.getIndentSpaces = void 0;
function getIndentSpaces(level = 0) {
    return new Array(2 * level).fill(' ').join('');
}
exports.getIndentSpaces = getIndentSpaces;
function createNamedImports(names, from) {
    return `import { ${names.join(', ')} } from '${from}';`;
}
exports.createNamedImports = createNamedImports;
function createSource(statements) {
    return statements.join('\n');
}
exports.createSource = createSource;
function createGrpcServiceSource(packageDefinition, staticObjectsRelativeFilename) {
    const [firstPackage] = packageDefinition.packages;
    return `
/* eslint-disable */
import { AbstractClientBase, GrpcWebClientBase, Metadata, Error, ClientReadableStream } from 'grpc-web';
import { ${firstPackage.name} } from '${staticObjectsRelativeFilename}';

type Options = {
  ignoreInterceptors?: boolean
}

export class GrpcService {
  private client: GrpcWebClientBase;
  private metadata: Metadata = {};
  private hostname: string;
  private interceptingPromise?: Promise<any>;
  public interceptors: { errors: ((e: any) => Promise<any>)[] } = {
    errors: []
  };
  constructor(hostname: string) {
    this.client = new GrpcWebClientBase({});
    this.hostname = hostname;
  }
  private makeInterceptedUnaryCall = <Result, Params, MethodInfo>(command: string, params: Params, methodInfo: MethodInfo, options: Options = {}): Promise<Result> => {
    const unaryCallHandler = (): Promise<Result> => this.client.unaryCall(command, params, this.metadata, methodInfo)
    
    if (options.ignoreInterceptors) {
      return unaryCallHandler()
    }
    
    if (this.interceptingPromise) {
      return this.interceptingPromise.then(() => {
        this.interceptingPromise = undefined;
        return unaryCallHandler()
      });
    }
    
    return new Promise((resolve, reject) => {
      unaryCallHandler().catch(e => {
        this.chainingInterceptors(this.interceptors.errors, e).then(() => {
          this.makeInterceptedUnaryCall<Result, Params, MethodInfo>(command, params, methodInfo).then(resolve).catch(reject)
        })
      });
    });
  }
  private chainingInterceptors = (interceptors: ((e: any) => Promise<any>)[], value: any) => {
    this.interceptingPromise = interceptors.reduce(
      (chain, nextInterceptor) => chain.then(nextInterceptor),
      Promise.resolve(value)
    );
    return this.interceptingPromise;
  }
  public setMetadata = (metadata: Metadata = {}) => {
    this.metadata = Object.assign({}, this.metadata, metadata);
  };
  ${firstPackage.services
        .map((service) => createServiceSource(service, firstPackage.name))
        .join(`\n${getIndentSpaces(1)}`)}
};`;
}
exports.createGrpcServiceSource = createGrpcServiceSource;
function createServiceSource(service, packageName) {
    return [
        `public ${service.name} = {`,
        `  ${service.methods
            .map((method) => createServiceMethodSource(method, service.name, packageName))
            .join(`\n${getIndentSpaces(2)}`)}`,
        '};',
    ].join(`\n${getIndentSpaces(1)}`);
}
exports.createServiceSource = createServiceSource;
function createServerStreamingFunction(packageName, serviceName, methodName) {
    return `return this.client.serverStreaming(this.hostname + '/${packageName}.${serviceName}/${methodName}', params, this.metadata, methodInfo)`;
}
function createServiceMethodSource(method, serviceName, packageName) {
    if (method.responseStream) {
        return [
            `${method.name}: (params: ${packageName}.I${method.requestType}): ClientReadableStream<${packageName}.${method.responseType}> => {`,
            '  const methodInfo = new AbstractClientBase.MethodInfo(',
            `    ${packageName}.${method.responseType},`,
            `    (request: ${packageName}.${method.requestType}) => ${packageName}.${method.requestType}.encode(request).finish(),`,
            `    ${packageName}.${method.responseType}.decode`,
            '  );',
            `  ${createServerStreamingFunction(packageName, serviceName, method.name)}`,
            '},',
        ].join(`\n${getIndentSpaces(2)}`);
    }
    return [
        `${method.name}: (params: ${packageName}.I${method.requestType}, options: Options = {}): Promise<${packageName}.${method.responseType}> => {`,
        '  const methodInfo = new AbstractClientBase.MethodInfo(',
        `    ${packageName}.${method.responseType},`,
        `    (request: ${packageName}.${method.requestType}) => ${packageName}.${method.requestType}.encode(request).finish(),`,
        `    ${packageName}.${method.responseType}.decode`,
        '  );',
        `  return this.makeInterceptedUnaryCall(this.hostname + '/${packageName}.${serviceName}/${method.name}', params, methodInfo, options);`,
        '},',
    ].join(`\n${getIndentSpaces(2)}`);
}
exports.createServiceMethodSource = createServiceMethodSource;
